#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<stack>
using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public UsedTypes<Use<MyVertex>   ::AsVertexType,
                                           Use<MyEdge>     ::AsEdgeType,
                                           Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face<   MyUsedTypes, face::FFAdj,  face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<   MyUsedTypes> {};

class MyMesh    : public tri::TriMesh< std::vector<MyVertex>, 
    std::vector<MyFace> , std::vector<MyEdge>  > {};

typedef face::Pos<MyFace> MyPos;

int main( int argc, char **argv )
{
  MyMesh m;
  
  tri::io::ImporterOFF<MyMesh>::Open(m, argv[1]);
  
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  
  MyPos startPos(&m.face[0], 0);
  
  std::stack<MyPos> toVisit;
  
  int count = 0;
  for (int i = 0; i < 3; ++i)
  {
    startPos.V()->SetV();
    count++;
    toVisit.push(startPos);
    startPos.FlipE();
    startPos.FlipV();
  }
  
  
  while (!toVisit.empty())
  {
    MyPos curr = toVisit.top();
    toVisit.pop();
    
    curr.FlipF();
    curr.FlipE();
    curr.FlipV();
    
    if (!curr.V()->IsV())
    {
      curr.V()->SetV();
      count++;
      toVisit.push(curr);
      curr.FlipE();
      curr.FlipV();
      toVisit.push(curr);
    }
  }
  
  printf("%i, %i\n", m.VN(), count);
  
  
  
    
  tri::io::ExporterOFF<MyMesh>::Save(m,"out.off");
  
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  
  return 0;
}
